unit uintermediariocostofuturo;

{$mode delphi}

interface

uses
  Classes, SysUtils,xmatdefs,uestados,umipsimplex,uCosaConNombre;


type

{ TIntermediarioCostoFuturo }

TIntermediarioCostoFuturo = class (TCosaConNombre)

  gradiente,estadoActual:TDAofNReal;
  idxCF,idxSPX:Array of integer; // vectores con los indices de CF y Simplex
  ivarCFSPX,iresCFSPX:integer; // indice del CF_Xs en el simplex en variable y en restriccion
  NVariablesDeEstado:integer;
  costoFuturoDelEstadoActual:NReal;
  constructor Create(capa: integer; const nombre: string);

  function getNombreVar(ivar:integer;var nombre: string): boolean;
  function getNombreRes(ires: integer; var nombre: string): boolean;
  // registra los indices de una variable de estado en el Costo Futuro y en el Simplex
  // estos tres procedimientos tienen que llamarse en ese orden
  procedure registrarVariables(iCF,iSPX:integer);
  // trae el gradiente del Costo Futuro en kPaso
  procedure getGradiente(CF: TAdminEstados; kPaso: integer);
  procedure getEstadoActual(CF: TAdminEstados; kPaso: integer);
  // cagra el gradiente en en simplex segun los inices definidos en idxCF e idxSPX
  procedure cargarGradienteEnSimplex(s:TMIPSimplex);
end;






implementation

{ TIntermediarioCostoFuturo }

constructor TIntermediarioCostoFuturo.Create(capa: integer; const nombre: string
  );
begin
  inherited create;
  self.nombre:=nombre;
end;

function TIntermediarioCostoFuturo.getNombreVar(ivar: integer;
  var nombre: string): boolean;
begin
  if ivar = ivarCFSPX then
  begin
    nombre:= self.Nombre;
    result:= True;
  end
  else
  begin
    nombre:= '';
    result:= False;
  end;
end;

function TIntermediarioCostoFuturo.getNombreRes(ires: integer;
  var nombre: string): boolean;
begin
  if ires = iresCFSPX then
  begin
    nombre:= self.Nombre;
    result:= True;
  end
  else
  begin
    nombre:= '';
    result:= False;
  end;
end;

procedure TIntermediarioCostoFuturo.registrarVariables(iCF, iSPX: integer);
begin
  // agarrar el costo futuro y casarlo con el indice de la variable dinamica del actor
  SetLength(idxCF,Length(idxCF)+1);
  SetLength(idxSPX,Length(idxSPX)+1);
  SetLength(gradiente,Length(gradiente)+1);
  SetLength(estadoActual,Length(estadoActual)+1);
  idxCF[High(idxCF)]:=iCF;
  idxSPX[High(idxSPX)]:=iSPX;
  NVariablesDeEstado:=Length(idxCF);
end;

procedure TIntermediarioCostoFuturo.getGradiente(CF: TAdminEstados;kPaso:integer);
var
  kVariableDeEstado,resCod:integer;
  dCdx_Inc,dCdx_Dec,xrpos:NReal;
begin
  for kVariableDeEstado:=0 to NVariablesDeEstado - 1 do
  begin
    CF.devxr_continuo(idxCF[kVariableDeEstado],kPaso,dCdx_Inc,dCdx_Dec,resCod,xrpos);
    gradiente[kVariableDeEstado]:=dCdx_Dec;
  end;
end;

procedure TIntermediarioCostoFuturo.getEstadoActual(CF: TAdminEstados;
  kPaso: integer);
var
  kVariableDeEstado:integer;
begin
  for kVariableDeEstado:=0 to NVariablesDeEstado - 1 do
  begin
    estadoActual[kVariableDeEstado]:=CF.xr[kVariableDeEstado];
  end;
  costoFuturoDelEstadoActual:=CF.costoContinuo(kPaso);
end;

procedure TIntermediarioCostoFuturo.cargarGradienteEnSimplex(s: TMIPSimplex);
var
  kVariableDeEstado:integer;

begin
  for kVariableDeEstado:=0 to NVariablesDeEstado - 1 do
  begin
    s.pon_e(s.nf - 1, idxSPX[kVariableDeEstado], -1*gradiente[kVariableDeEstado]);
    s.acum_e(s.nf - 1, s.nc - 1,gradiente[kVariableDeEstado] *
      estadoActual[kVariableDeEstado]);
  end;
  s.pon_e(s.nf - 1, s.nc - 1, 1);
  s.acum_e(s.nf - 1, s.nc - 1,-1 * costoFuturoDelEstadoActual);
end;

end.

