unit unubeseable;

{$mode delphi}

interface

(**
   Esta unidad administra el Almacen de Cosas tanto en el CacheLocal como en la Nube.

   Las Cosas que puede almacenar son una cadena de caracteres identificadas por
   un nid_cosa que identifica la Instancia de la cosa y el nid_version que identifica
   la versión de la instancia de la cosa.

**)

uses
  urosx,
  uDataSetGenerico, xmatdefs, usqlutils,
  Classes, SysUtils;

const
  // Códigos de error
  EC_fallo_conexion_url_nube = -1;
  EC_no_encontrada_en_la_nube = -2;
  EC_no_encontrada_en_cache = -3;
  EC_no_encontrada_en_ningun_lado = -4;
  EC_usuario_no_autenticado = -5;
  EC_usuario_o_clave_no_validos = -6;

  // Ruta de cache
  rutaCache = 'c:\simsee\basura\data_simsee';


type

  TRecPermisosNube = class

  end;

  { TNubeAdmin }

  TNubeAdmin = class
    id_hilo: integer;
    cache_dir: string;
    nube_url: string;
    PASAPORTE: string; // obtenido en el login
    nid_usr: integer;
    nid_grupos: TDAofNInt;

    constructor Create(id_hilo: integer; cache_dir, nube_url: string);

    // retorna TRUE si fue exitoso e inicializa el PASAPORTE
    // Llama a la función login del script url_nube y obtiene
    // un pasaporte válido.
    // Todas las funciones contra la nube deben pasar el parámetro
    // psp=PASAPORTE  que será chequedo por el script url_nube antes
    // de proceder a ejecutar.
    function Login(usuario, clave: string): boolean;

    // Llama a la función logut de url_nube y cancela la validéz del
    // PASAPORTE. Vacía la variable PASAPORTE indicando que no está logueado.
    function logout: boolean;

    function Permisos(nid_data: integer): TRecPermisosNube;


    // Inserta una nueva instancia de una cosa y retorna el nid_cosa asignado
    // si tiene éxito. Si se produce un error retorna 0 (cero).
    // La primer instancia de una cosa tiene nid_version = 1
    function WriteToNube(Data: string): integer;

    // baja de la nube el Data correspondiente a (nid, nid_version)
    // retorna vacío si falló y en codErr el error.
    function ReadFromNube(nid_data, nid_version: integer;
      var codErr: integer): string;

    function WriteToCache(nid_data, nid_version: integer; const Data: string): boolean;
    function ReadFromCache(nid_data, nid_version: integer;
      var codErr: integer): string;

    // retornan la última versión ya sea de la nube o del cache y -1 si no encuentran
    function UltimaVersionFromNube(nid_data: integer): integer;
    function UltimaVersionFromCache(nid_data: integer): integer;

    // Escribe a la nube nueva versión del Data (nid_data) y si tienen éxito,
    // retorna un entero positivo con el número de versión asignado.
    // Si hay error retoran un negativo con el código de error.
    function WriteNewVersionToNube(nid_data: integer; const Data: string): integer;

    // retorna el path completo al archivo en el cache (cache_dir)/(nid_data)_v(nid_version).txt
    function ArchiCache(nid_data, nid_version: integer): string;

    // Elimina todo los objetos de cache
    function ClearCache: integer;

    // Elimina todos los objetos de cache asociados a un (nid_data)
    function ClearCacheOf(nid_data: integer): integer;

    // si lo tiene en el cache lo lee de cache sino lo busca en la nube
    // si falla retorna el string vacío y en errCod retorna el código de error
    // si tiene éxito la variable errCod se retora en CERO.
    function GetData(nid_data, nid_version: integer; var errCod: integer): string;

  private
    function GetPasaporte(cant_letras: integer): string;
  end;

var
  nubeAdmin: TNubeAdmin;
  dbcon_rosx: TDBrosxCon;

// retorna mensaje de error leible por humano.
function ecToMsg(errCod: integer): string;


implementation

function CrearSiNoExiste(carpeta: string): string;
begin
  if not DirectoryExists(carpeta) then
    try
      mkdir(carpeta)
    except
      on E: Exception do
        raise Exception.Create('Error al intentar crear la carpeta: ' + carpeta);
    end;
  Result := carpeta;
end;

// retorna mensaje de error leible por humano.
function ecToMsg(errCod: integer): string;
begin
  case errCod of
    EC_fallo_conexion_url_nube: Result := 'Falló conexión a la url_nube';
    EC_no_encontrada_en_la_nube: Result := 'Cosa no encontrada en la nube';
    EC_no_encontrada_en_cache: Result := 'Cosa no encontrada en el cache';
    EC_no_encontrada_en_ningun_lado: Result :=
        'Cosa no encontrada ni en la nube, ni en cache';
    EC_usuario_no_autenticado: Result := 'Usuario no autenticado';
    EC_usuario_o_clave_no_validos: Result := 'Usuario o clave no validos';
    else
      Result := 'ErrCod: ' + IntToStr(errCod) + ' DESCONOCIDO!.';
  end;
end;

procedure InsertNube(memo: string);
var
  sql: string;
begin

  //nid_version, usuario, grupo
  sql := 'INSERT INTO nube_simsee (dt_creacion,comentarios,data_memo) VALUES (' +
    QuotedStr(DateTimeToStr(now)) + ',' + QuotedStr('prueba') + ',' + QuotedStr(memo) + ')';

  writeln(sql);


  // dbcon_rosx.Exec(sql);

end;



{ TNubeAdmin }

constructor TNubeAdmin.Create(id_hilo: integer; cache_dir, nube_url: string);
begin
  self.id_hilo := id_hilo;
  self.cache_dir := cache_dir;
  self.nube_url := nube_url;
end;

function TNubeAdmin.Login(usuario, clave: string): boolean;
var
  sql: string;
  ds: TResultadoQuery;
  r: TDataRecord;
begin

  sql := 'SELECT * FROM usuarios_simsee WHERE nombre=' + QuotedStr(usuario) +
    ' AND clave=' + QuotedStr(clave);

  ds := dbcon_rosx.sql_query(sql);

  if ds.EOF then
    Result := False
  else
  begin
    r := ds.First;
    self.PASAPORTE := GetPasaporte(5); //obtiene un pasaporte de 5 digitos
    Result := True;
  end;

end;

function TNubeAdmin.logout: boolean;
begin

end;

function TNubeAdmin.Permisos(nid_data: integer): TRecPermisosNube;
begin

end;

function TNubeAdmin.WriteToNube(Data: string): integer;
var
  sql: string;
  ds: TResultadoQuery;
  r: TDataRecord;
  nid_version: string;
  th: TSQLTableHelper;
  res: integer;
begin
  //nid_version, usuario, grupo

  nid_version := '1'; //cuando se incerta por primera vez vale 1.
  th := TSQLTableHelper.Create('nube_simsee');
  th.add('nid_version', nid_version);
  th.add('dt_creacion', DateTimeToStr(now));
  th.add('comentarios', 'nombre del actor');
  th.add('data_memo', Data);


  sql := th.GetInsertStr(False);

  res := dbcon_rosx.Exec(sql);

  if res = -1 then
    Result := 0
  else
  begin
    sql := 'SELECT nid FROM nube_simsee ORDER BY nid DESC';

    ds := dbcon_rosx.sql_query(sql);
    r := ds.First;

    Result := r.GetByNameAsInt('nid');
  end;

end;

function TNubeAdmin.ReadFromNube(nid_data, nid_version: integer;
  var codErr: integer): string;
var
  sql: string;
  ds: TResultadoQuery;
  r: TDataRecord;
begin

  //nid_version, usuario, grupo
  sql := 'SELECT * FROM nube_simsee WHERE nid=' + IntToStr(nid_data) +
    ' AND nid_version=' + IntToStr(nid_version);

  ds := dbcon_rosx.sql_query(sql);

  if ds.EOF then
  begin
    codErr := EC_no_encontrada_en_la_nube;
    Result := '';
  end
  else
  begin
    r := ds.First;
    codErr := 0;  //0: ok
    Result := r.GetByNameAsString('data_memo');

  end;
end;

function TNubeAdmin.WriteToCache(nid_data, nid_version: integer;
  const Data: string): boolean;
var
  f: TextFile;
  archi: string;
begin
  archi := self.ArchiCache(nid_data, nid_version);
  if not fileexists(archi) then
  begin
    AssignFile(f, archi);
    Rewrite(f);
    WriteLn(f, Data);
    CloseFile(f);
  end;

end;

function TNubeAdmin.ReadFromCache(nid_data, nid_version: integer;
  var codErr: integer): string;
var
  f: TextFile;
  archi, linea, memo: string;
begin
  archi := self.ArchiCache(nid_data, nid_version);
  memo:= '';
  if fileexists(archi) then
  begin
    AssignFile(f, archi);
    reset(f);
    while not EOF(f) do
    begin
      readLn(f, linea);
      memo := memo + linea + #13;
    end;
    Close(f);
    Result := memo;
    codErr := 0;
  end
  else
  begin
    codErr := EC_no_encontrada_en_cache;
    Result := '';
  end;
end;

function TNubeAdmin.UltimaVersionFromNube(nid_data: integer): integer;

var
  sql: string;
  ds: TResultadoQuery;
  r: TDataRecord;

begin

  sql := 'SELECT * FROM nube_simsee WHERE nid=' + IntToStr(nid_data) +
    ' ORDER BY nid_version LIMIT 1';

  ds := dbcon_rosx.sql_query(sql);

  if ds.EOF then
    Result := -1 //no hay última versión del archivo solicitado en la nube
  else
  begin
    r := ds.First;
    Result := r.GetByNameAsInt('nid_version');
  end;

end;

function TNubeAdmin.UltimaVersionFromCache(nid_data: integer): integer;
var
  memo: string;
  knid_version, codErr: integer;
begin
  memo := '';
  knid_version := 0;
  codErr := 0;

  while codErr = 0 do
  begin
    Inc(knid_version);
    memo := self.ReadFromCache(nid_data, knid_version, codErr);
  end;

  if knid_version = 1 then
    Result := -1
  else
    Result := knid_version - 1;

end;

function TNubeAdmin.WriteNewVersionToNube(nid_data: integer;
  const Data: string): integer;
var
  sql: string;
  ds: TResultadoQuery;
  r: TDataRecord;
  nid_version: integer;
  th: TSQLTableHelper;
begin

  //Ultima versión
  nid_version := self.UltimaVersionFromNube(nid_data);
  nid_version := nid_version + 1; //uno mas de la que había

  th := TSQLTableHelper.Create('nube_simsee');
  th.add('nid_version', nid_version);
  th.add('dt_creacion', DateTimeToStr(now));
  th.add('comentarios', 'nombre del actor');
  th.add('data_memo', Data);

  sql := th.GetInsertStr(False);

  if dbcon_rosx.Exec(sql) = -1 then
    Result := EC_fallo_conexion_url_nube
  else
    Result := nid_version;

end;

function TNubeAdmin.ArchiCache(nid_data, nid_version: integer): string;
var
  carpeta_data, archivo: string;

begin

  carpeta_data := CrearSiNoExiste(rutaCache);
  archivo := carpeta_data + DirectorySeparator + IntToStr(nid_data) + '_v' +
    IntToStr(nid_version) + '.txt';

  Result := archivo;
end;



function TNubeAdmin.ClearCache: integer;
var
  SR: TSearchRec;
begin
  if FindFirst(rutaCache + '*.*', faArchive+faHidden+faReadOnly, SR)= 0 then
   repeat
     DeleteFile(rutaCache+'\'+SR.Name);
   until FindNext(SR) <> 0;
 Result := 0; // que devuelvo??
end;

function TNubeAdmin.ClearCacheOf(nid_data: integer): integer;
var
  archi: string;
  ult_version, k : integer;
begin

  ult_version := self.UltimaVersionFromCache(nid_data);

  for k:=1 to ult_version do
  begin
    archi := self.ArchiCache(nid_data, k);
    if fileexists(archi) then
    begin
      DeleteFile(archi);
    end;
  end;

  Result := 0; // que devuelvo??

end;

function TNubeAdmin.GetData(nid_data, nid_version: integer;
  var errCod: integer): string;
var
  NoHayCache: boolean;
  memo: string;

begin
  memo := '';
  //fijarse primero en el cache si el actor se encuentra allí, sino
  //bajar de la nube

  memo := ReadFromCache(nid_data, nid_version, errCod);
  if errCod = EC_no_encontrada_en_cache then
    memo := ReadFromNube(nid_data, nid_version, errCod);

  if errCod = EC_no_encontrada_en_la_nube then
    errCod := EC_no_encontrada_en_ningun_lado;

  Result := memo;

end;

function TNubeAdmin.GetPasaporte(cant_letras: integer): string;
var
  i: integer;
  res: string;
begin


end;



initialization
{$IFDEF NUBE}
  nubeAdmin := TNubeAdmin.Create(0, 'C:\SimSEE\basura', '/zeros/x/rosx.php');
  dbcon_rosx := TDBrosxCon.Create('', 'latorre.adme.com.uy', 80, '/zeros/x/rosx.php');
  dbcon_rosx.determinar_tipo_servidor;
{$ENDIF}

finalization
{$IFDEF NUBE}
  if dbcon_rosx <> nil then
    dbcon_rosx.Free;
{$ENDIF}
end.

